@@ -4,8 +4,8 @@
  **   Modules Revision 3.0						     **
  **   Providing a flexible user environment				     **
  ** 									     **
- **   File:		ModuleCmd_Avail.c				     **
- **   First Edition:	91/10/23					     **
+ **   File:		Modulate_Avail.c				     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Authors:	John Furlan, jlf@behere.com				     **
  **		Jens Hamisch, jens@Strawberry.COM			     **
@@ -34,7 +34,7 @@
  ** 									     ** 
  ** ************************************************************************ **/
 
-static char Id[] = "@(#)$Id: ModuleCmd_Avail.c,v 1.2 2001/06/09 09:48:46 rkowen Exp $";
+static char Id[] = "@(#)$Id: ModuleCmd_Avail.c,v 1.10 2005/11/29 04:26:30 rkowen Exp $";
 static void *UseId[] = { &UseId, Id };
 
 /** ************************************************************************ **/
@@ -90,8 +90,9 @@ typedef struct _subdir_node {
 #ifdef CACHE_AVAIL
 static	char	*namebuf = NULL;
 #endif
+static	char	 buffer[MOD_BUFSIZE];
 static	char	 buf[ LINELENGTH];
-static	char	module_name[] = "ModuleCmd_Avail.c";	/** File name of this module **/
+static	char	 module_name[] = "ModuleCmd_Avail.c";	/** File name of this module **/
 
 #if WITH_DEBUGGING_MODULECMD
 static	char	_proc_ModuleCmd_Avail[] = "ModuleCmd_Avail";
@@ -171,7 +172,7 @@ static	char	**create_cache_list( FILE*, int*, char* );
  ** 									     **
  **   Description:	Execution of the 'module avail' command		     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	Tcl_Interp	*interp		Current Tcl Interpr. **
  **			char		*argv[]		Arguments to the     **
@@ -180,7 +181,7 @@ static	char	**create_cache_list( FILE*, int*, char* );
  **   Result:		int	TCL_OK		Successfull operation	     **
  **				TCL_ERROR	Any failure		     **
  ** 									     **
- **   Attached Globals:	specified_module	The module name from the     **
+ **   Attached Globals:	g_specified_module	The module name from the     **
  **						command line.		     **
  ** 									     **
  ** ************************************************************************ **
@@ -192,7 +193,7 @@ int ModuleCmd_Avail(	Tcl_Interp	*interp,
 {
     char	*dirname;
     char	*modpath;
-    int		 Result = TCL_OK;
+    int		 Result = -TCL_ERROR;
     
 #if WITH_DEBUGGING_MODULECMD
     ErrorLogger( NO_ERR_START, LOC, _proc_ModuleCmd_Avail, NULL);
@@ -203,20 +204,14 @@ int ModuleCmd_Avail(	Tcl_Interp	*interp,
      **  We perform 1 level of env.var. expansion
      **/
 
-    if( !(modpath = (char *) xgetenv( "MODULEPATH"))) {
+    if( !(modpath = (char *) xgetenv( "MODULEPATH")))
 	if( OK != ErrorLogger( ERR_MODULE_PATH, LOC, NULL))
-	    return( TCL_ERROR);		/** --- EXIT PROCEDURE (FAILURE) --> **/
-    }
+	    goto unwind0;
  
 #ifdef CACHE_AVAIL
-    if( !(namebuf = (char*) malloc( MOD_BUFSIZE * sizeof( char)))) {
-	if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL)) {
-	    free( modpath);
-	    free( buf);
-	    return( TCL_ERROR);		/** --- EXIT PROCEDURE (FAILURE) --> **/
-	}
-    }
-
+    if( (char *) NULL == (namebuf = stringer(NULL, MOD_BUFSIZE, NULL)))
+	if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL))
+	    goto unwind1;
 #endif
 
     /**
@@ -232,7 +227,7 @@ int ModuleCmd_Avail(	Tcl_Interp	*interp,
 	     ** Set the name of the module specified on the command line
 	     **/
 
-	    specified_module = *argv;
+	    g_specified_module = *argv;
 
 	    if( !check_dir( *argv))
 		if( OK != ErrorLogger( ERR_PARAM, LOC, NULL)) {
@@ -265,7 +260,7 @@ int ModuleCmd_Avail(	Tcl_Interp	*interp,
 		 ** Set the name of the module specified on the command line
 		 **/
 
-		specified_module = *argv;
+		g_specified_module = *argv;
 
 		dirname = modpath;
 		while( dirname && *dirname) {
@@ -293,7 +288,7 @@ int ModuleCmd_Avail(	Tcl_Interp	*interp,
 	    }
 
 	/**
-	 **  Otherwise, if there's no cathegorie given, descend the current
+	 **  Otherwise, if there's no category given, descend the current
 	 **  directory and print its contents.
 	 **/
 
@@ -327,15 +322,25 @@ int ModuleCmd_Avail(	Tcl_Interp	*interp,
     /**
      **  Free up what has been allocated and exit from this procedure
      **/
+    /* if got here via this path ... it must have been OK */
+    if(Result < 0) Result = TCL_OK;
 
-    free( modpath);
-    free( namebuf);
+unwind2:
+#ifdef CACHE_AVAIL
+    null_free((void *) &namebuf);
+#endif
+unwind1:
+    null_free((void *) &modpath);
 
+unwind0:
 #if WITH_DEBUGGING_MODULECMD
     ErrorLogger( NO_ERR_END, LOC, _proc_ModuleCmd_Avail, NULL);
 #endif
 
-    return( Result);
+    /* if Result is negative here ... must have been an unwind */
+    if (Result < 0) Result = -Result;
+
+    return( Result);		/** --- EXIT PROCEDURE (FAILURE/SUCCESS) --> **/
 
 } /** End of 'ModuleCmd_Avail' **/
 
@@ -347,7 +352,7 @@ int ModuleCmd_Avail(	Tcl_Interp	*interp,
  **   Description:	Open and close the passed directory in order to check**
  **			if it does exist and is readable		     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	char	*dirname	Name of the directory to be  **
  **						checked			     **
@@ -386,7 +391,7 @@ int check_dir(	char	*dirname)
  ** 									     **
  **   Description:	Print all files beyond the passed directory	     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	char	*dir		Directory to be scanned	     **
  **			char	*module		A selcted module name or NULL**
@@ -444,17 +449,12 @@ static	int	print_dir(	Tcl_Interp	*interp,
     if( module) {
 
 	if( dir) {
-	    if((char *) NULL == (selection = (char *) malloc( dirlen +
-		strlen( module) + 1))) {
-		ErrorLogger( ERR_ALLOC, LOC, NULL);
+	    if((char *) NULL == (selection = stringer(NULL, 0,
+		dir,"/",module, NULL))) {
+		ErrorLogger( ERR_STRING, LOC, NULL);
 		return( TCL_ERROR);     /** --- EXIT (FAILURE) --------> **/
 	    }
 
-	    /* sprintf( selection, "%s/%s", dir, module); */
-	    strcpy( selection, dir);
-	    strcat( selection, "/");
-	    strcat( selection, module);
-
 	} else
 	    selection = module;
 
@@ -479,16 +479,14 @@ static	int	print_dir(	Tcl_Interp	*interp,
 
 	if( !sw_create) {
 
-	    /* sprintf( namebuf, "%s/.moduleavailcache", dir); */
-	    strcpy( namebuf, dir);
-	    strcat( namebuf, "/.moduleavailcache");
+	    if( (char *) NULL == stringer(namebuf, MOD_BUFSIZE,
+		dir, "/.moduleavailcache", NULL))
+		if( OK != ErrorLogger( ERR_STRING, LOC, NULL))
+		    goto unwind1;
+
 	    if( NULL == (fi = fopen( namebuf, "r"))) {
-		if( OK != ErrorLogger( ERR_OPEN, LOC, namebuf, "reading",
-		    NULL)) {
-		    if( selection && dir)
-			free( selection);
-		    return( TCL_ERROR);     /** ----- EXIT (FAILURE) ------> **/
-		}
+		if( OK != ErrorLogger( ERR_OPEN, LOC, namebuf, "reading", NULL))
+		    goto unwind1;
 
 	    } else {
 
@@ -499,11 +497,8 @@ static	int	print_dir(	Tcl_Interp	*interp,
 		 **/
 
 		if( EOF == fclose( fi))
-		    if( OK != ErrorLogger( ERR_CLOSE, LOC, namebuf, NULL)) {
-			if( selection && dir)
-			    free( selection);
-			return( TCL_ERROR);	/** --- EXIT (FAILURE) ----> **/
-		    }
+		    if( OK != ErrorLogger( ERR_CLOSE, LOC, namebuf, NULL))
+			goto unwind1;
 	    }
 
 	} /** if( !create) **/
@@ -521,18 +516,12 @@ static	int	print_dir(	Tcl_Interp	*interp,
 	 **/
 
 	if( NULL == (dirlst_head = get_dir( dir, NULL, &count, &tcount)))
-	    if( OK != ErrorLogger( ERR_READDIR, LOC, dir, NULL)) {
-		if( selection && dir)
-		    free( selection);
-		return( TCL_ERROR);     /** ------- EXIT (FAILURE) --------> **/
-	    }
+	    if( OK != ErrorLogger( ERR_READDIR, LOC, dir, NULL))
+		goto unwind1;
 
 	if( NULL == (cache_list = (char**) malloc( tcount * sizeof( char**))))
-	    if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL)) {
-		if( selection && dir)
-		    free( selection);
-		return( TCL_ERROR);     /** ------- EXIT (FAILURE) --------> **/
-	    }
+	    if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL))
+		goto unwind1;
 	(void) memset(cache_list, 0, tcount * sizeof( char **));
 
 	start=0;
@@ -579,9 +568,8 @@ static	int	print_dir(	Tcl_Interp	*interp,
 	SourceVers( interp, dir, module);
     }
 
-    if( dir && selection && selection) {
-	free( selection);
-    }
+    if( dir && selection)
+	null_free((void *) &selection);
 
     /**
      **  Print and remove the cache list
@@ -594,7 +582,14 @@ static	int	print_dir(	Tcl_Interp	*interp,
 
     if( sw_format & SW_LONG)
 	fprintf( stderr, "\n");
-    return( TCL_OK);
+    return( TCL_OK);     		/** ------- EXIT (SUCCESS) --------> **/
+
+unwind1:
+    if( dir && selection)
+	null_free((void *) &selection);
+
+unwind0:
+    return( TCL_ERROR);     		/** ------- EXIT (FAILURE) --------> **/
 
 } /** End of 'print_dir' **/
 
@@ -606,7 +601,7 @@ static	int	print_dir(	Tcl_Interp	*interp,
  **   Description:	Check whether an avail cache exists and is not out   **
  **			of date						     **
  ** 									     **
- **   First Edition:	96/01/03					     **
+ **   First Edition:	1996/01/03					     **
  ** 									     **
  **   Parameters:	char	*dir		Directory to be checked	     **
  ** 									     **
@@ -631,9 +626,9 @@ static	int	check_cache( char *dir)
      **  younger than the related directory.
      **/
 
-    /* sprintf( namebuf, "%s/.moduleavailcachedir", dir); */
-    strcpy( namebuf, dir);
-    strcat( namebuf, "/.moduleavailcachedir");
+    if( (char *) NULL == stringer(namebuf, MOD_BUFSIZE,
+	dir, "/.moduleavailcachedir", NULL))
+            return( 0);
 
     if( NULL != (cdir = fopen( namebuf, "r"))) {
         if( stat( dir, &dir_stats) != -1) {
@@ -682,6 +677,9 @@ static	int	check_cache( char *dir)
 #endif
 
 
+static int	test_version_dir(	struct dirent	*dp)
+{
+}
 /*++++
  ** ** Function-Header ***************************************************** **
  ** 									     **
@@ -689,8 +687,11 @@ static	int	check_cache( char *dir)
  ** 									     **
  **   Description:	Read in the passed directory and save every interes- **
  **			ting item in the directory list			     **
+ **			skipping known version control directories:	     **
+ **				CVS RCS .svn				     **
+ **			unless they contain .version files		     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	char	*dir		Directory to be read	     **
  **			char	*prefix		Directory prefix (path)	     **
@@ -750,21 +751,21 @@ fi_ent	*get_dir(	char	*dir,
 	if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL)) {
 	    if( -1 == closedir( dirptr))
 		ErrorLogger( ERR_CLOSEDIR, LOC, dir, NULL);
-	    return( NULL);	/** ----------- EXIT (FAILURE) ------------> **/
+	    goto unwind0;
 	}
     }
     dirlst_last = dirlst_head + DIREST;
   
     /**
-     **  Read in the  contents of the directory. Ignore dotfiles.
+     **  Read in the  contents of the directory. Ignore dotfiles
+     **		and version directories.
      **/
 
     for( count = 0,  dp = readdir( dirptr); dp != NULL; dp = readdir( dirptr)) {
-        if( *dp->d_name == '.')
-	    continue;
+        if( *dp->d_name == '.') continue;
 
 	/**
-	 **  Conditionally double up the space allocated foe reading the direc-
+	 **  Conditionally double up the space allocated for reading the direc-
 	 **  tory
 	 **/
 
@@ -772,7 +773,7 @@ fi_ent	*get_dir(	char	*dir,
             if( NULL == (dirlst_head = (fi_ent*) realloc( (char*) dirlst_head, 
 		(count<<1) * sizeof( fi_ent)))) 
 		if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL)) 
-		    return( NULL);	/** ------- EXIT (FAILURE) --------> **/
+		    goto unwind0;
             dirlst_cur = dirlst_head + count;
             dirlst_last = dirlst_head + (count<<1);
 	}
@@ -781,23 +782,19 @@ fi_ent	*get_dir(	char	*dir,
 	 **  Build the complete path name and get information about the file
 	 **/
 
-        if( !( dirname = mkdirnm( dir, dp->d_name))) {
+        if( !( dirname = mkdirnm( dir, dp->d_name)))
 	    if( OK != ErrorLogger( ERR_DIRNAME, LOC, NULL)) {
 		if( -1 == closedir( dirptr))
 		    ErrorLogger( ERR_CLOSEDIR, LOC, dir, NULL);
-		free( dirlst_cur);
-		return( NULL);	/** ----------- EXIT (FAILURE) ------------> **/
+		goto unwind1;
 	    }
-	}
 
-        if( stat( dirname, &(dirlst_cur->fi_stats)) < 0) {
+        if( stat( dirname, &(dirlst_cur->fi_stats)) < 0)
 	    if( OK != ErrorLogger( ERR_DIRNOTFOUND, LOC, dirname, NULL)) {
 		if( -1 == closedir( dirptr))
 		    ErrorLogger( ERR_CLOSEDIR, LOC, dir, NULL);
-		free( dirlst_cur);
-		return( NULL);	/** ----------- EXIT (FAILURE) ------------> **/
+		goto unwind1;
 	    }
-        }
 
 	/**
 	 **  If it is a directory, recursively delve into it ..
@@ -816,33 +813,52 @@ fi_ent	*get_dir(	char	*dir,
 		if( OK != ErrorLogger( ERR_DIRNAME, LOC, NULL)) {
 		    if( -1 == closedir( dirptr))
 			ErrorLogger( ERR_CLOSEDIR, LOC, dir, NULL);
-		    free( dirlst_cur);
-		    return( NULL);	/** ------- EXIT (FAILURE) --------> **/
+		    goto unwind1;
 		}
 	    } else {
 		if( NULL == (np = strdup( tmp)))
 		    if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL))
-			return( NULL);	/** ------- EXIT (FAILURE) --------> **/
+			goto unwind1;
 	    }
 
             if( !( tmp = mkdirnm( dir, dp->d_name))) {
 		if( OK != ErrorLogger( ERR_DIRNAME, LOC, NULL)) {
 		    if( -1 == closedir( dirptr))
 			ErrorLogger( ERR_CLOSEDIR, LOC, dir, NULL);
-		    free( dirlst_cur);
-		    return( NULL);	/** ------- EXIT (FAILURE) --------> **/
+		    goto unwind1;
 		}
 	    } else {
 		if( NULL == (ndir = strdup( tmp)))
 		    if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL)) 
-			return( NULL);	/** ------- EXIT (FAILURE) --------> **/
+			goto unwind1;
 	    }
 
 	    /**
-	     **  The recursion itsself ...
+	     **  What if it's a known version control directory
+	     **  check if it has a .version file
+	     **/
+	    if (!strcmp("CVS",dp->d_name)
+	    ||  !strcmp("RCS",dp->d_name)
+	    ||  !strcmp(".svn",dp->d_name)) {
+    		FILE	*fi;
+		if( (char *) NULL == stringer(buffer, MOD_BUFSIZE,
+		    tmp, "/.version", NULL))
+		    if( OK != ErrorLogger( ERR_STRING, LOC, NULL))
+			goto unwind1;
+		if( NULL == (fi = fopen( buffer, "r"))) {
+			/* does not have a .version file */
+			continue;
+		} else {
+			/* has a .version file ... assume to be module dir */
+			fclose(fi);
+		}
+	    }
+
+	    /**
+	     **  The recursion itself ...
 	     **/
 
-            dirlst_cur->fi_subdir = get_dir( ndir, np, &dirlst_cur->fi_listcount, 
+            dirlst_cur->fi_subdir = get_dir( ndir,np,&dirlst_cur->fi_listcount,
 		&tmpcount);
 
             /**
@@ -859,8 +875,8 @@ fi_ent	*get_dir(	char	*dir,
              **/
 
             if( !dirlst_cur->fi_listcount)
-		free( np);
-            free( ndir);
+		null_free((void *) &np);
+            null_free((void *) &ndir);
 
 	/**
 	 **  if it is not a directory check the magic cookie of the file. Only
@@ -883,7 +899,7 @@ fi_ent	*get_dir(	char	*dir,
 	dirlst_cur->fi_prefix = prefix;
 	if( NULL == (dirlst_cur->fi_name = strdup( dp->d_name)))
 	    if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL)) 
-		return( NULL);		/** ------- EXIT (FAILURE) --------> **/
+		goto unwind1;
 
 	/**
 	 **  Count even the number of elements in the current list as the
@@ -905,16 +921,21 @@ fi_ent	*get_dir(	char	*dir,
 	qsort( dirlst_head, count, sizeof(fi_ent), fi_ent_cmp);
 
     /**
-     **  Close the directory, set upt return values
+     **  Close the directory, set up return values
      **/
 
     if( -1 == closedir( dirptr))
 	if( OK != ErrorLogger( ERR_CLOSEDIR, LOC, dir, NULL))
-	    return( NULL);
+	    goto unwind1;
 
     *listcount = count;
-    return( dirlst_head);
+    return( dirlst_head); 	/** ----------- EXIT (SUCCESS) ------------> **/
 
+unwind1:
+    null_free((void *) &dirlst_cur);
+
+unwind0:
+    return( NULL);	 	/** ----------- EXIT (FAILURE) ------------> **/
 } /** End of 'get_dir' **/
 
 /*++++
@@ -925,7 +946,7 @@ fi_ent	*get_dir(	char	*dir,
  **   Description:	Transform the passed nested directory list into a    **
  **			flat list of strings				     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	char	**list		List to be created	     **
  **			fi_ent	 *dirlst_head	Head of the directory list   **
@@ -980,28 +1001,22 @@ void	dirlst_to_list(	char	**list,
         if( dirlst_cur->fi_prefix) {
 
 	    if( path) {
-		/* sprintf( buf, "%s/%s/%s", path, dirlst_cur->fi_prefix,
-		    dirlst_cur->fi_name); */
-		strcpy( buf, path);
-		strcat( buf, "/");
-		strcat( buf, dirlst_cur->fi_prefix);
-		strcat( buf, "/");
-		strcat( buf, dirlst_cur->fi_name);
+		if( (char *) NULL == stringer(buf, MOD_BUFSIZE,
+		    path,"/", dirlst_cur->fi_prefix,"/", dirlst_cur->fi_name,
+		    NULL))
+			return;
 	    } else {
-		/* sprintf( buf, "%s/%s", dirlst_cur->fi_prefix,
-		    dirlst_cur->fi_name); */
-		strcpy( buf, dirlst_cur->fi_prefix);
-		strcat( buf, "/");
-		strcat( buf, dirlst_cur->fi_name);
+		if( (char *) NULL == stringer(buf, MOD_BUFSIZE,
+		    dirlst_cur->fi_prefix,"/", dirlst_cur->fi_name, NULL))
+			return;
 	    }
 
 	    ptr = buf;
         } else {
 	    if( path) {
-		/* sprintf( buf, "%s/%s", path, dirlst_cur->fi_name); */
-		strcpy( buf, path);
-		strcat( buf, "/");
-		strcat( buf, dirlst_cur->fi_name);
+		if( (char *) NULL == stringer(buf, MOD_BUFSIZE,
+		    path,"/", dirlst_cur->fi_name, NULL))
+			return;
 		ptr = buf;
 
 	    } else
@@ -1021,7 +1036,7 @@ void	dirlst_to_list(	char	**list,
 	    if( NULL == (list[(*beginning)++] = strdup( ptr))) {
 		if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL)) {
 		    while( i--) 
-			free( list[--(*beginning)]);
+			null_free((void *) list + (--(*beginning)));
 		    return;		/** ------- EXIT (FAILURE) --------> **/
 		}
 	    }
@@ -1046,7 +1061,7 @@ void	dirlst_to_list(	char	**list,
  **   Description:	Delete an entire directory list including all sub-   **
  **			directory lists					     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	fi_ent	*dirlst_head	Head of the list to be re-   **
  **						moved			     **
@@ -1079,7 +1094,7 @@ void	delete_dirlst(	fi_ent	*dirlst_head,
 	 i<count && dirlst_cur;
          i++, dirlst_cur++) {
 
-        free( dirlst_cur->fi_name);
+        null_free((void *) &(dirlst_cur->fi_name));
 
 	/**
 	 **  Recursivle decend to subdirectories
@@ -1094,8 +1109,8 @@ void	delete_dirlst(	fi_ent	*dirlst_head,
      **/
 
     if( dirlst_head->fi_prefix)
-	free( dirlst_head->fi_prefix);
-    free((char*) dirlst_head);
+	null_free((void *) &(dirlst_head->fi_prefix));
+    null_free((void *) &dirlst_head);
 
 } /** End of 'delete_dirlst' **/
 
@@ -1106,7 +1121,7 @@ void	delete_dirlst(	fi_ent	*dirlst_head,
  ** 									     **
  **   Description:	Write a cache file using the given directory list    **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters: 	fi_ent	*dirlist_head	List of files to be printed  **
  **			int	 count		Number of entries in the pas-**
@@ -1141,15 +1156,17 @@ static	void	store_files(	fi_ent	*dirlst_head,
      **  Open both, the cache info and the cache dir file
      **/
 
-    /* sprintf( namebuf, "%s/.moduleavailcache", dir); */	/** CACHEINFO   **/
-    strcpy( namebuf, dir);					/** CACHEINFO   **/
-    strcat( namebuf, "/.moduleavailcache");
+    /** CACHEINFO   **/
+    if( (char *) NULL == stringer(namebuf, MOD_BUFSIZE,
+	dir, "/.moduleavailcache", NULL))
+            return;
     if( NULL == (fi = fopen( namebuf, "w+")))
 	return;
 
-    /* sprintf( namebuf, "%s/.moduleavailcachedir", dir); */	/** CACHEOUTPUT **/
-    strcpy( namebuf, dir);					/** CACHEOUTPUT **/
-    strcat( namebuf, "/.moduleavailcachedir");
+    /** CACHEOUTPUT **/
+    if( (char *) NULL == stringer(namebuf, MOD_BUFSIZE,
+	dir, "/.moduleavailcachedir", NULL))
+            return;
     if( NULL == (cdir = fopen( namebuf, "w+"))) {
 	if( EOF == fclose( fi))
 	    ErrorLogger( ERR_CLOSE, LOC, NULL);
@@ -1184,7 +1201,7 @@ static	void	store_files(	fi_ent	*dirlst_head,
  ** 									     **
  **   Description:	Write the contents of a cache file		     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	FILE	*cacheoutput	Output stream to be used     **
  **			FILE	*cacheinfo	Cache log file to be written **
@@ -1246,13 +1263,13 @@ static	void	store_dirlst(	FILE	*cacheinfo,
     } /** for **/
 
     /**
-     **  Free up everything that has been allocated foe the directory
+     **  Free up everything that has been allocated for the directory
      **  list
      **/
 
     if( dirlst_head->fi_prefix)
-	free( dirlst_head->fi_prefix);
-    free((char*) dirlst_head);
+	null_free((void *) &(dirlst_head->fi_prefix));
+    null_free((void *) &dirlst_head);
 
 } /** End of 'store_dirlst' **/
 
@@ -1264,7 +1281,7 @@ static	void	store_dirlst(	FILE	*cacheinfo,
  **   Description:	Store the name of the file passed as 'file entry' to **
  **			the specified output stream if it isn't a temp file  **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	FILE	*cacheoutput	Output stream to be used     **
  **			char	*dir		The current directory	     **
@@ -1310,7 +1327,7 @@ static	void	store_file(	FILE	*cacheoutput,
      **  Finally free up the memory that has been used to store the filename
      **/
 
-    free( file->fi_name);
+    null_free((void *) &(file->fi_name));
 
 } /** End of 'store_file' **/
 
@@ -1323,7 +1340,7 @@ static	void	store_file(	FILE	*cacheoutput,
  **   Description:	Read the passed cache-file and create a list of file-**
  **			names out of it					     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	FILE	*cacheinput	Opened cache file	     **
  **			int	*count		Buffer to save the number of **
@@ -1402,8 +1419,8 @@ static	char	**create_cache_list(	FILE	*cacheinput,
 	if( 1 != fscanf( cacheinput, "%s", buf)) {
 	    if( OK != ErrorLogger( ERR_READ, LOC, "cache", NULL)) {
 		while( --i)
-		    free( list[ i]);
-		free( list);
+		    null_free((void *) list + i);
+		null_free((void *) &list);
 		return( NULL);	/** ----------- EXIT (I/O error) ----------> **/
 	    }
 	}
@@ -1424,7 +1441,7 @@ static	char	**create_cache_list(	FILE	*cacheinput,
         if( NULL == (list[ i] = strdup( buf))) {
 	    if( OK != ErrorLogger( ERR_ALLOC, LOC, NULL)) {
 		while( --i)
-		    free( list[ i]);
+		    null_free((void *) list + i);
 		return( NULL);  /** ------------ EXIT (FAILURE) -----------> **/
 	    }
 	}
@@ -1448,7 +1465,7 @@ static	char	**create_cache_list(	FILE	*cacheinput,
  **   Description:	Remove an entire list of allocated strings and free  **
  **			up the used memory				     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	char	**list		List of filenames to be print**
  **			int	  tcount	Size ofd the list in elements**
@@ -1470,9 +1487,9 @@ void delete_cache_list(	char	**list,
 #endif
 
     for( i=0; i<tcount; i++)
-        free( list[i]);
+        null_free((void *) (list + i));
     
-    free((char*) list);
+    null_free((void *)&list);
 
 } /** End of 'delete_cache_list' **/
 
@@ -1485,7 +1502,7 @@ void delete_cache_list(	char	**list,
  **			column by column taking care of the order being re-  **
  **			flected to the single columns			     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	char	**list		List of filenames to print   **
  **			char	 *path		common path		     **
@@ -1711,7 +1728,7 @@ void print_aligned_files(	Tcl_Interp	 *interp,
 
 	    }
 
-	    free( module);
+	    null_free((void *) &module);
 
 	} /** if( !stat) **/
 
@@ -1729,7 +1746,7 @@ void print_aligned_files(	Tcl_Interp	 *interp,
     }
 
     if (! modpath) {
-	free(modpath);
+	null_free((void *)&modpath);
     }
 
 #if WITH_DEBUGGING_UTIL_1
@@ -1746,7 +1763,7 @@ void print_aligned_files(	Tcl_Interp	 *interp,
  **   Description:	Print out the filenames in the _file_list array in   **
  **			case of terse output 				     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	int	  terminal_width	Terminal size	     **
  **			int	  len			max. filename length **
@@ -1852,7 +1869,7 @@ static	void	print_terse_files(  int terminal_width,
  ** 									     **
  **   Description:	File list functions for terse module display mode    **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	char	*name		Name to be stored	     **
  ** 									     **
@@ -1866,8 +1883,7 @@ static	void	print_terse_files(  int terminal_width,
 static	void _init_file_list()
 {
     if( _file_list_ptr && !_file_list_cnt) {
-	free( _file_list_ptr);
-	_file_list_ptr = (char **) NULL;
+	null_free((void *) &_file_list_ptr);
 	_file_list_cnt = 0;
     }
 
@@ -1933,7 +1949,7 @@ static	char *_pick_file_list( int ndx)
  **   Description:	Print out the passed filename and fill the output    **
  **			area up to the passed number of characters	     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	char	*name		Name to be printed	     **
  **			int	 maxwidth	With of the output field to  **
@@ -1996,7 +2012,7 @@ static	void print_spaced_file(	char	*name,
  **   Description:	Build a full pathname out of the passed directory    **
  **			and file					     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	char	*dir		The directory to be used     **
  **			char	*file		The filename w/o path	     **
@@ -2024,7 +2040,7 @@ static	char *mkdirnm(	char	*dir,
      **  return just the file.
      **/
 
-    if( dir == NULL || *dir == '\0' || *dir == '.')
+    if( dir == NULL || *dir == '\0' || !strcmp(dir,"."))
 	return( strcpy( dirbuf, file));
 
     /**
@@ -2061,7 +2077,7 @@ static	char *mkdirnm(	char	*dir,
  **   Notes:		This procedure is used as comparison function for    **
  **			qsort()					 	     **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	const void *fi1		First file entry	     **
  **			const void *fi2		Second one to compare	     **
@@ -2078,5 +2094,10 @@ static	char *mkdirnm(	char	*dir,
 static	int fi_ent_cmp(	const void	*fi1,
 			const void	*fi2)
 {
+
+#ifdef DEF_COLLATE_BY_NUMBER
+  return colcomp( ((fi_ent*)fi1)->fi_name, ((fi_ent*)fi2)->fi_name);
+#else
   return strcmp( ((fi_ent*)fi1)->fi_name, ((fi_ent*)fi2)->fi_name);
+#endif
 }
