@@ -5,7 +5,7 @@
  **   Providing a flexible user environment				     **
  ** 									     **
  **   File:		ModuleCmd_Load.c				     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Authors:	John Furlan, jlf@behere.com				     **
  **		Jens Hamisch, jens@Strawberry.COM			     **
@@ -28,7 +28,7 @@
  ** 									     ** 
  ** ************************************************************************ **/
 
-static char Id[] = "@(#)$Id: ModuleCmd_Load.c,v 1.5 2002/01/04 04:59:14 rkowen Exp $";
+static char Id[] = "@(#)$Id: ModuleCmd_Load.c,v 1.8.6.2 2006/02/21 22:26:56 rkowen Exp $";
 static void *UseId[] = { &UseId, Id };
 
 /** ************************************************************************ **/
@@ -79,7 +79,7 @@ static	char	_proc_ModuleCmd_Load[] = "ModuleCmd_Load";
  ** 									     **
  **   Description:	Execution of the module-commands 'load' and 'unload' **
  ** 									     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Parameters:	Tcl_Interp	*interp		Attached Tcl Interp. **
  **			int		 load		Controls 'load' or   **
@@ -92,7 +92,7 @@ static	char	_proc_ModuleCmd_Load[] = "ModuleCmd_Load";
  **				1		At least one module has been **
  **						read			     **
  ** 									     **
- **   Attached Globals:	specified_module	The module name from the     **
+ **   Attached Globals:	g_specified_module	The module name from the     **
  **						command line		     **
  **   			g_flags		These are set up accordingly before  **
  **					this function is called in order to  **
@@ -116,6 +116,7 @@ int	ModuleCmd_Load(	Tcl_Interp	*interp,
     Tcl_Interp		 *tmp_interp;		/** Tcl interpreter to be    **/
 						/** used internally	     **/
     Tcl_HashTable	**oldTables = NULL;
+    EM_RetVal		  em_return_val = EM_OK;
 
 #if WITH_DEBUGGING_MODULECMD
     ErrorLogger( NO_ERR_START, LOC, _proc_ModuleCmd_Load, NULL);
@@ -139,7 +140,7 @@ int	ModuleCmd_Load(	Tcl_Interp	*interp,
 	 ** Set the name of the module specified on the command line
 	 **/
 
-	specified_module = argv[i];
+	g_specified_module = argv[i];
 
 	/**
 	 **  Create a Tcl interpreter and initialize it with the module commands
@@ -171,8 +172,10 @@ int	ModuleCmd_Load(	Tcl_Interp	*interp,
 		 **  So it is loaded ...
 		 **  Do we know the filename?
 		 **/
-
-                strcpy( modulename, tmpname);
+                if ((char *) NULL == stringer(modulename, MOD_BUFSIZE,
+			tmpname, NULL))
+		    if( OK != ErrorLogger( ERR_STRING, LOC, NULL))
+			goto unwind0;
                 if( !filename[0])
                     if( TCL_ERROR == (return_val = Locate_ModuleFile(
 			tmp_interp, argv[i], tmpname, filename))) 
@@ -182,8 +185,8 @@ int	ModuleCmd_Load(	Tcl_Interp	*interp,
                  **  If IsLoaded() created tmpname, then we must free it.
                  **/
 
-                if( tmpname && tmpname != argv[i]) 
-                    free( tmpname);
+                if( tmpname && (tmpname != argv[i]))
+                    null_free((void *) &tmpname);
 
             } /** if loaded **/
 
@@ -191,14 +194,11 @@ int	ModuleCmd_Load(	Tcl_Interp	*interp,
 	 **  LOAD to be done
 	 **  Only check the filename
 	 **/
-
         } else {
-
 	    if( TCL_ERROR == (return_val = Locate_ModuleFile( tmp_interp,
 		argv[i], modulename, filename)))
 		ErrorLogger( ERR_LOCATE, LOC, argv[i], NULL);
         }
-
         /**
          **  If return_val has been set to something other than TCL_OK,
          **  then read_status should not be set -- by the ANSI definition.
@@ -210,35 +210,51 @@ int	ModuleCmd_Load(	Tcl_Interp	*interp,
 
 	g_current_module = modulename;
 	if( TCL_OK == return_val) {
-            return_val = Read_Modulefile( tmp_interp, filename);
+	    return_val = Read_Modulefile(tmp_interp, filename);
+	    em_return_val = ReturnValue(tmp_interp, return_val);
 
-            switch (return_val) {
-              case TCL_OK:
-                /**
-                 ** If module terminates TCL_OK, add it to the loaded list...
-                 **/
+	    switch (em_return_val) {
+	    case EM_OK:
+		/**
+		 ** If module terminates TCL_OK, add it to the loaded list ...
+		 **/
+	    case EM_CONTINUE:
+		/**
+		 ** If module terminates TCL_CONTINUE, add it to the
+		 ** loaded list ... but further processing of that module
+		 ** has ceased
+		 **/
+		Tcl_ResetResult(tmp_interp);
 		Update_LoadedList( tmp_interp, modulename, filename);
 
-              case TCL_BREAK:
-                /**
-                 ** If module terminates TCL_BREAK, don't add it to the list,
-                 ** but assume that everything was OK with the module anyway.
-                 **/
 		/**
 		 **  Save the current environment setup before the next module
 		 **  file is (un)loaded in case something is broken ...
 		 **  ... for Unwind_Modulefile_Changes later on
 		 **/
+		Tcl_ResetResult(tmp_interp);
         	if( oldTables) {
                     Delete_Hash_Tables( oldTables);
-                    free((char*) oldTables);
+                    null_free((void *) &oldTables);
         	}
         	oldTables = Copy_Hash_Tables();
 		a_successful_load = 1;
-                break;  /* switch */
+		break;	/* switch */
 
-              case TCL_ERROR:
-              default:
+	    case EM_BREAK:
+		/**
+		 ** If module terminates TCL_BREAK, don't add it to the list,
+		 ** but assume that everything was OK with the module anyway.
+		 ** (The original code wasn't correct)
+		 **/
+	    case EM_EXIT:
+		/**
+		 ** If module terminates TCL_EXIT, find the return value
+		 ** and exit modulecmd with that ... also none of the
+		 ** following modules should be loaded.
+		 **/
+	    case EM_ERROR:
+	    default:
 		/**
 		 **  Reset what has been changed.
 		 **/
@@ -246,28 +262,28 @@ int	ModuleCmd_Load(	Tcl_Interp	*interp,
             
         	oldTables = NULL;
 		return_val = TCL_ERROR;
-                break;  /* switch */
+		break;	/* switch */
 	    }
 	}
-
         Tcl_DeleteInterp(tmp_interp);
 
+	/* do not process any further modulefiles if EM_EXIT */
+	if (em_return_val == EM_EXIT) break; /* for */
+
     } /** for **/
-    
+
     /**
      **  There may only be a spare save environment left, if the final module
      **  has been load successfully. Remove it in this case
      **/
-
-    if( return_val == TCL_OK && oldTables) {
+    if( em_return_val == EM_OK && oldTables) {
         Delete_Hash_Tables( oldTables);
-        free((char*) oldTables);
+        null_free((void *) &oldTables);
     }
 
     /**
      **  Clean up the flags and return 
      **/
-
     if( load)
         g_flags &= ~M_LOAD;
     else
@@ -279,4 +295,7 @@ int	ModuleCmd_Load(	Tcl_Interp	*interp,
 
     return( a_successful_load);
 
+unwind0:
+    return( TCL_ERROR);			/** -------- EXIT (FAILURE) -------> **/
+
 } /** End of 'ModuleCmd_Load' **/
