@@ -5,7 +5,7 @@
  **   Providing a flexible user environment				     **
  ** 									     **
  **   File:		cmdTrace.c					     **
- **   First Edition:	95/12/26					     **
+ **   First Edition:	1995/12/26					     **
  ** 									     **
  **   Authors:	Jens Hamisch, jens@Strawberry.COM			     **
  ** 									     **
@@ -29,7 +29,7 @@
  ** 									     ** 
  ** ************************************************************************ **/
 
-static char Id[] = "@(#)$Id: cmdTrace.c,v 1.2 2001/06/09 09:48:46 rkowen Exp $";
+static char Id[] = "@(#)$Id: cmdTrace.c,v 1.8 2005/11/29 04:26:30 rkowen Exp $";
 static void *UseId[] = { &UseId, Id };
 
 /** ************************************************************************ **/
@@ -138,6 +138,12 @@ typedef	struct	_mod_trace	{
 #   define	MOD_TR_APROPOS	_all_off
 #endif
 
+#ifdef  WITH_TRACE_REFRESH
+#   define	MOD_TR_REFRESH	WITH_TRACE_REFRESH
+#else
+#   define	MOD_TR_REFRESH	_all_off
+#endif
+
 #ifdef  WITH_TRACE_WHATIS
 #   define	MOD_TR_WHATIS	WITH_TRACE_WHATIS
 #else
@@ -191,8 +197,10 @@ static	ModTrace	TraceSelect[] = {
     { &purgeRE,		"purge",	MOD_TR_PURGE,	0 },	/** 11 **/
     { &clearRE,		"clear",	MOD_TR_CLEAR,	0 },	/** 12 **/
     { &whatisRE,	"whatis",	MOD_TR_WHATIS,	0 },	/** 13 **/
-    { &aproposRE,	"apropos",	MOD_TR_APROPOS,	0 }	/** 14 **/
+    { &aproposRE,	"apropos",	MOD_TR_APROPOS,	0 },	/** 14 **/
+    { &refreshRE,	"refresh",	MOD_TR_REFRESH,	0 }	/** 15 **/
 };
+/** ************************************************************************ **/
 
 /** ************************************************************************ **/
 /**				    PROTOTYPES				     **/
@@ -219,7 +227,7 @@ static	int	CheckTracingPat(	Tcl_Interp *interp,
  ** 									     **
  **   Description:	Callback function for 'trace'			     **
  ** 									     **
- **   First Edition:	95/12/26					     **
+ **   First Edition:	1995/12/26					     **
  ** 									     **
  **   Parameters:	ClientData	 client_data			     **
  **			Tcl_Interp	*interp		According Tcl interp.**
@@ -240,7 +248,7 @@ static	int	CheckTracingPat(	Tcl_Interp *interp,
 int	cmdModuleTrace(	ClientData	 client_data,
 	      		Tcl_Interp	*interp,
 	      		int		 argc,
-	      		char		*argv[])
+	      		CONST84 char	*argv[])
 {
     char 	  on_off = '+';		/** The first argument		     **/
     char	**args;			/** Argument pointer for scanning    **/
@@ -256,14 +264,12 @@ int	cmdModuleTrace(	ClientData	 client_data,
     /**
      **  Whatis mode?
      **/
-
     if( g_flags & (M_WHATIS | M_HELP))
         return( TCL_OK);		/** ------- EXIT PROCEDURE -------> **/
 	
     /**
      **  Parameter check
      **/
-
     if( argc < 2) {
 	if( OK != ErrorLogger( ERR_USAGE, LOC, argv[0], "on|off",
 	    "[cmd [cmd ...]]", "[ -module module [module ...]]", NULL))
@@ -273,7 +279,6 @@ int	cmdModuleTrace(	ClientData	 client_data,
     /**
      **  On or off?
      **/
-
     if( !strcmp( argv[ 1], "on"))
 	on_off = '+';
     else if( !strcmp( argv[ 1], "off"))
@@ -284,10 +289,18 @@ int	cmdModuleTrace(	ClientData	 client_data,
 	    return( TCL_ERROR);		/** -------- EXIT (FAILURE) -------> **/
     }
  
+  
+    /**
+     **  Non-persist mode?
+     **/
+    
+    if (g_flags & M_NONPERSIST) {
+	return (TCL_OK);
+    }
+
     /**
      **  Display mode?
      **/
-
     if( g_flags & M_DISPLAY) {
 	fprintf( stderr, "%s\t ", argv[ 0]);
 	for( i=1; i<argc; i++)
@@ -300,7 +313,6 @@ int	cmdModuleTrace(	ClientData	 client_data,
      **  We need a table of module command involved in this trace selection.
      **  Allocate one and initialize it.
      **/
-
     cmd_tab_size = sizeof( TraceSelect) / sizeof( TraceSelect[ 0]);
     if((char *) NULL == (cmd_table = (char *) malloc( cmd_tab_size)))
 	return((OK == ErrorLogger( ERR_ALLOC, LOC, NULL)) ? TCL_OK : TCL_ERROR);
@@ -309,8 +321,7 @@ int	cmdModuleTrace(	ClientData	 client_data,
      **  Scan all commands specified as options. The last option to be scanned
      **  is either the real last one, or the '-module' one
      **/
-
-    args = argv + 2; i = argc - 2;
+    args = (char **) argv + 2; i = argc - 2;
     memset( cmd_table, !i, cmd_tab_size);
 
     while( i--) {
@@ -319,7 +330,6 @@ int	cmdModuleTrace(	ClientData	 client_data,
 	/**
 	 **  Check for '-module'
 	 **/
-
 	if( !strncmp( tmp, "-module", strlen( tmp)))
 	    break;
 
@@ -327,12 +337,11 @@ int	cmdModuleTrace(	ClientData	 client_data,
 	 **  This should be a module command.
 	 **  Check it against the TraceSelect table
 	 **/
-
 	if( -1 != (k = GetTraceTable(interp, tmp, cmd_tab_size))) {
 	    cmd_table[ k] = 1;
 	} else {
 	    if( OK != ErrorLogger( ERR_COMMAND, LOC, tmp, NULL)) {
-		free( cmd_table);
+		null_free((void *) &cmd_table);
 		return( TCL_ERROR);
 	    }
 	}
@@ -344,7 +353,6 @@ int	cmdModuleTrace(	ClientData	 client_data,
      **  If we ran to the end of the argument list (i==0), ALL files are
      **  concerned in this ...
      **/
-
     if( 0 >= i) {	
 	ret = ChangeTraceSel(interp, cmd_table, cmd_tab_size, on_off, _all);
     } else {
@@ -356,8 +364,7 @@ int	cmdModuleTrace(	ClientData	 client_data,
     /**
      **  Cleanup finally and return
      **/
-
-    free( cmd_table);
+    null_free((void *) &cmd_table);
 
 #if WITH_DEBUGGING_CALLBACK
     ErrorLogger( NO_ERR_END, LOC, _proc_cmdModuleTrace, NULL);
@@ -375,7 +382,7 @@ int	cmdModuleTrace(	ClientData	 client_data,
  **   Description:	Returns the TraceSelect index for the passed module  **
  **			subcommand					     **
  ** 									     **
- **   First Edition:	95/12/26					     **
+ **   First Edition:	1995/12/26					     **
  ** 									     **
  **   Parameters:	char	*cmd		Subcommand to be checked     **
  **			int	 num		Number of commands to be chk **
@@ -398,7 +405,6 @@ static	int	GetTraceTable(Tcl_Interp *interp, char *cmd, int num)
     /**
      **  Not found ..
      **/
-
     return( -1);
 
 } /** End of 'GetTraceTable' **/
@@ -411,7 +417,7 @@ static	int	GetTraceTable(Tcl_Interp *interp, char *cmd, int num)
  **   Description:	Retrieve the trace selection pattern for the passed  **
  **			module command					     **
  ** 									     **
- **   First Edition:	95/12/26					     **
+ **   First Edition:	1995/12/26					     **
  ** 									     **
  **   Parameters:	char	*cmd		Subcommand to be checked     **
  ** 									     **
@@ -443,7 +449,7 @@ char	*GetTraceSel(	Tcl_Interp *interp,
  **			fied in the passed 'cmd_table'. The passed module-   **
  **			name has to be changed according 'on_off'	     **
  ** 									     **
- **   First Edition:	95/12/26					     **
+ **   First Edition:	1995/12/26					     **
  ** 									     **
  **   Parameters:	char	*cmd_table	Boolean array indicating all **
  **						commands in the TraceSelect  **
@@ -474,7 +480,6 @@ static	int	ChangeTraceSel(	Tcl_Interp *interp,
     /**
      **  Need a buffer for to build the complete pattern
      **/
-
     if((char *) NULL == (pattern = (char *) malloc( len + 2))) {
 	ErrorLogger( ERR_ALLOC, LOC, NULL);
 	return( TCL_ERROR);
@@ -484,14 +489,13 @@ static	int	ChangeTraceSel(	Tcl_Interp *interp,
      **  Check if this is the ALL pattern. If it is, replace all affected
      **  entries with '_all_on' or '_all_off'
      **/
-
     if( !strcmp( module_pat, _all)) {
 
 	for( i=0; i < cmd_tab_size; i++) {
 	    if( cmd_table[ i]) {
 
 		if( TraceSelect[ i].alloc)
-		    free( TraceSelect[ i].tracing)
+		    null_free((void *) &(TraceSelect[ i].tracing));
 		TraceSelect[ i].alloc = 0;
 
 		TraceSelect[ i].tracing = ('+' == on_off) ?
@@ -504,7 +508,6 @@ static	int	ChangeTraceSel(	Tcl_Interp *interp,
 	/**
 	 **  Check the pattern for colons ...
 	 **/
-
 	if( strchr( module_pat, ':')) 
 	    if( OK != ErrorLogger( ERR_COLON, LOC, module_pat, NULL))
 		ret = TCL_ERROR;
@@ -514,7 +517,6 @@ static	int	ChangeTraceSel(	Tcl_Interp *interp,
 	    /**
 	     **  Build the complete pattern
 	     **/
-
 	    *pattern = on_off;
 	    strcpy( pattern + 1, module_pat);
 	    len += 1;
@@ -522,14 +524,12 @@ static	int	ChangeTraceSel(	Tcl_Interp *interp,
 	    /**
 	     **  Loop for all entries to be changed
 	     **/
-
 	    for( i=0; i < cmd_tab_size; i++) {
 		if( cmd_table[ i]) {
 
 		    /**
 		     **  allocate a buffer for the new pattern
 		     **/
-		    
 		    if((char *) NULL == (tmp = (char *) malloc( len + 2 +
 			strlen( TraceSelect[ i].tracing)))) {
 			if( OK == ErrorLogger( ERR_ALLOC, LOC, NULL)) {
@@ -543,14 +543,12 @@ static	int	ChangeTraceSel(	Tcl_Interp *interp,
 		    /**
 		     **  Copy the new pattern to the buffer at first
 		     **/
-
 		    strcpy( tmp, pattern);
 		    t = tmp + len;
 
 		    /**
 		     **  Tokenize the old pattern and remove duplicates
 		     **/
-
 		    for( s = strtok( TraceSelect[ i].tracing, ":");
 			 s;
 			 s = strtok( NULL, ":") ) {
@@ -569,7 +567,7 @@ static	int	ChangeTraceSel(	Tcl_Interp *interp,
 		     **/
 
 		    if( TraceSelect[ i].alloc)
-			free( TraceSelect[ i].tracing);
+			null_free((void *) &(TraceSelect[ i].tracing));
 
 		    TraceSelect[ i].tracing = tmp;
 		    TraceSelect[ i].alloc = 1;
@@ -583,8 +581,7 @@ static	int	ChangeTraceSel(	Tcl_Interp *interp,
     /**
      **  Free what has been allocated an return 
      **/
-
-    free( pattern);
+    null_free((void *) &pattern);
     return( ret);
 
 } /** End of 'ChangeTraceSel' **/
@@ -597,7 +594,7 @@ static	int	ChangeTraceSel(	Tcl_Interp *interp,
  **   Description:	Check wheter thracing is turned on for the passed    **
  **			command and modulefile				     **
  ** 									     **
- **   First Edition:	95/12/26					     **
+ **   First Edition:	1995/12/26					     **
  ** 									     **
  **   Parameters:	char	*cmd		Subcommand to be checked     **
  **			char	*modulefile	Modulefile to be checked     **
@@ -640,7 +637,7 @@ int	CheckTracing(	Tcl_Interp *interp,
  **   Description:	Check the passed pattern if it enables tracing for   **
  **			the passed module file				     **
  ** 									     **
- **   First Edition:	95/12/26					     **
+ **   First Edition:	1995/12/26					     **
  ** 									     **
  **   Parameters:	char	*pattern	Pattern to be checked	     **
  **			char	*modulefile	Modulefile to be checked     **
@@ -688,7 +685,7 @@ static	int	CheckTracingPat(	Tcl_Interp *interp,
  **   Description:	Check wheter tracing is turned on for the passed     **
  **			command and at least one of the passed modulefiles   **
  ** 									     **
- **   First Edition:	95/12/26					     **
+ **   First Edition:	1995/12/26					     **
  ** 									     **
  **   Parameters:	char	*cmd		Subcommand to be checked     **
  **			int	 count		Number of passed modulefiles **
