@@ -29,7 +29,7 @@
  ** 									     ** 
  ** ************************************************************************ **/
 
-static char Id[] = "@(#)$Id: main.c,v 1.16.4.2.4.3 2006/05/04 22:03:47 rkowen Exp $";
+static char Id[] = "@(#)$Id: main.c,v 1.16.4.2.4.3.2.2 2006/06/01 14:32:07 rkowen Exp $";
 static void *UseId[] = { &UseId, Id };
 
 /** ************************************************************************ **/
@@ -79,6 +79,7 @@ int	  g_flags = 0,			/** Control what to do at the moment **/
 	  append_flag = 0;		/** only used by the 'use' command   **/
 
 char	  _default[] = "default";	/** id string for default versions   **/
+char	  _colon[]   = ":";		/** directory separator		     **/
 
 /**
  **  Name of the rc files
@@ -154,6 +155,9 @@ static	char	module_name[] = "main.c";	/** File name of this module **/
 #if WITH_DEBUGGING
 static	char	_proc_main[] = "main";
 #endif
+#if WITH_DEBUGGING_MODULECMD
+static	char	_proc_Module_Usage[] = "Module_Usage";
+#endif
 #if WITH_DEBUGGING_INIT
 static	char	_proc_Check_Switches[] = "Check_Switches";
 static	char	_proc_Tcl_AppInit[] = "Tcl_AppInit";
@@ -384,7 +388,7 @@ void module_usage(FILE *output)
      **/
 
 #if WITH_DEBUGGING_MODULECMD
-    ErrorLogger( NO_ERR_START, LOC, _proc_ModuleCmd_Help, NULL);
+    ErrorLogger( NO_ERR_START, LOC, _proc_Module_Usage, NULL);
 #endif
 
 	fprintf(output,
