@@ -1,15 +1,15 @@
 /*****
  ** ** Module Header ******************************************************* **
  ** 									     **
- **   Modules Revision 3.0						     **
+ **   Modules Revision 3.2						     **
  **   Providing a flexible user environment				     **
  ** 									     **
  **   File:		modules_def.h					     **
- **   Revision:		1.21						     **
- **   First Edition:	91/10/23					     **
+ **   First Edition:	1991/10/23					     **
  ** 									     **
  **   Authors:	John Furlan, jlf@behere.com				     **
  **		Jens Hamisch, jens@Strawberry.COM			     **
+ **		R.K. Owen, rk@owen.sj.ca.us				     **
  ** 									     **
  **   Description:							     **
  ** 									     **
@@ -35,35 +35,51 @@
 /** ************************************************************************ **/
 
 #include <stdio.h>
+#include <stdarg.h>
 #include <tcl.h>
 #include "config.h"
 
+#ifndef CONST84
+#  define CONST84	/* const */
+#endif
+
+#if (TCL_MAJOR_VERSION < 8)
+#  define TCL_RESULT(interp) ((interp)->result)
+#else
+#  define TCL_RESULT(interp) Tcl_GetStringResult(interp)
+#endif
+
+#ifndef HAVE_STDINT_H
+/* assume 32 bit - hope for the best */
+typedef	int	intptr_h;
+#endif
+
 #ifdef HAVE_STDLIB_H
-#include <stdlib.h>
+#  include <stdlib.h>
 #endif
 
 #if STDC_HEADERS || HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 /**
  **  An ANSI string.h and pre-ANSI memory.h might conflict.
  **/
-#if !STDC_HEADERS && HAVE_MEMORY_H
-#include <memory.h>
-#endif /** not STDC_HEADERS and HAVE_MEMORY_H **/
+#  if !STDC_HEADERS && HAVE_MEMORY_H
+#    include <memory.h>
+#  endif /** not STDC_HEADERS and HAVE_MEMORY_H **/
 #else /** not STDC_HEADERS and not HAVE_STRING_H **/
-#include <strings.h>
+#  include <strings.h>
 /**
  **  memory.h and strings.h conflict on some systems.
  **/
 #endif /** not STDC_HEADERS and not HAVE_STRING_H  **/
 
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#include <sys/types.h>
+#  include <unistd.h>
+#  include <sys/types.h>
 #endif
 
 #ifdef HAVE_CTYPE_H
-#include <ctype.h>
+#  include <ctype.h>
 #endif
 
 #ifdef HAVE_SYS_MODE_H
@@ -71,44 +87,44 @@
 #endif
 
 #ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
+#  include <sys/stat.h>
 #endif
 
 #ifdef HAVE_SYS_TERMIOS_H
-#include <sys/termios.h>
+#  include <sys/termios.h>
 #else
-#ifdef HAVE_TERMIO_H
-#include <termio.h>
-#endif
+#  ifdef HAVE_TERMIO_H
+#    include <termio.h>
+#  endif
 #endif
 
 #ifdef HAVE_FCNTL_H
-#include <fcntl.h>
+#  include <fcntl.h>
 #endif
 
 #if !defined(TIOCGWINSZ) && defined(HAVE_SYS_IOCTL_H)
-#include <sys/ioctl.h>
+#  include <sys/ioctl.h>
 #endif
 
 #if defined(DIRENT) || defined(_POSIX_VERSION)
-#include <dirent.h>
-#define NLENGTH(dirent) (strlen((dirent)->d_name))
+#  include <dirent.h>
+#  define NLENGTH(dirent) (strlen((dirent)->d_name))
 #else /** not (DIRENT or _POSIX_VERSION) **/
-#define dirent direct
-#define NLENGTH(dirent) ((dirent)->d_namlen)
-#ifdef SYSNDIR
-#include <sys/ndir.h>
-#endif /** SYSNDIR **/
-#ifdef SYSDIR
-#include <sys/dir.h>
-#endif /** SYSDIR **/
-#ifdef NDIR
-#include <ndir.h>
-#endif /** NDIR **/
+#  define dirent direct
+#  define NLENGTH(dirent) ((dirent)->d_namlen)
+#  ifdef SYSNDIR
+#    include <sys/ndir.h>
+#  endif /** SYSNDIR **/
+#  ifdef SYSDIR
+#    include <sys/dir.h>
+#  endif /** SYSDIR **/
+#  ifdef NDIR
+#    include <ndir.h>
+#  endif /** NDIR **/
 #endif /** not (DIRENT or _POSIX_VERSION) **/
 
 #ifdef HAVE_ERRNO_H
-#include <errno.h>
+#  include <errno.h>
 #else
 extern	int	  errno;
 #endif
@@ -175,6 +191,7 @@ typedef	enum	{
 	ERR_UNAME,			/** Uname failed		     **/
 	ERR_GETHOSTNAME,		/** gethostname failed		     **/
 	ERR_GETDOMAINNAME,		/** getdomainname failed	     **/
+	ERR_STRING,			/** string error		     **/
 	ERR_DISPLAY = 90,		/** cannot open display	    	     **/
 	ERR_IN_MODULEFILE = 100,	/** modulefile related errors	     **/
 	ERR_PARSE,			/** Parse error (modulefile)	     **/
@@ -196,6 +213,8 @@ typedef	enum	{
 	ERR_DUP_ALIAS,			/** Duplicate alias		     **/
 	ERR_CACHE_INVAL,		/** Invalid cache version	     **/
 	ERR_CACHE_LOAD,			/** Cannot load cache properly	     **/
+	ERR_BEGINENV,			/** No update if no .modulesbeginenv **/
+	ERR_BEGINENVX,			/** No MODULESBEGINENV env.var.      **/
 	ERR_INIT_TCL,			/** Cannot initialize TCL	     **/
 	ERR_INIT_TCLX,			/** Cannot initialize extended TCL   **/
 	ERR_INIT_ALPATH,		/** Cannot set up autoload path      **/
@@ -210,7 +229,8 @@ typedef	enum	{
 	ERR_INTERRL,			/** Error logger internal error	     **/
 	ERR_INVAL,			/** Invalid parameter to the error   **/
 	ERR_INVWGHT,			/** logger			     **/
-	ERR_INVFAC			/** Invalid error facility	     **/
+	ERR_INVFAC,			/** Invalid error facility	     **/
+        ERR_ENVVAR,                     /** env. variables are inconsistent  **/
 } ErrType;
 
 /**
@@ -220,30 +240,36 @@ typedef	enum	{
  **/
 
 typedef	enum	{
-	OK = 0,				/** Everything's up and running	     **/
-	WARN = 2,			/** A warning (mapped to OK by the   **/
+	OK	= 0,			/** Everything's up and running	     **/
+	WARN	= 2,			/** A warning (mapped to OK by the   **/
 					/** Errorlogger			     **/
-	PROBLEM = 5,			/** Problem ... program might cont.  **/
-	ERROR = 7,			/** Error .. try gracefull aborting  **/
-	FATAL = 10,			/** The following will lead to the   **/
-	PANIC = 20			/** progrm being aborted by the er-  **/
+	PROBLEM	= 5,			/** Problem ... program might cont.  **/
+	ERROR	= 7,			/** Error .. try gracefull aborting  **/
+	FATAL	= 10,			/** The following will lead to the   **/
+	PANIC	= 20			/** progrm being aborted by the er-  **/
 					/** ror logger immediatelly	     **/
 } ErrCode;
 
+/**
+ **  Internal return value to handle the various ways a module load
+ **	could end.
+ **/
+typedef enum	{
+	EM_OK		= 0,		/** normal return	**/
+	EM_EXIT,			/** cmd: exit N		**/
+					/** (set g_retval = N)	**/
+	EM_BREAK,			/** cmd: break		**/
+	EM_CONTINUE,			/** cmd: continue	**/
+	EM_ERROR			/** abnormal return	**/
+} EM_RetVal;
+
 /** ************************************************************************ **/
 /** 				     CONSTANTS				     **/
 /** ************************************************************************ **/
 
-/**
- **   VERSION of MODULE
- **/
-
 #define      MODULES_MAGIC_COOKIE         "#%Module"
 #define      MODULES_MAGIC_COOKIE_LENGTH  8
 
-#define      MODULES_RELEASE       "3.0RKO"
-#define      MODULES_PATCHLEVEL    "0"
-
 /**
  **  User level
  **/
@@ -343,6 +369,7 @@ typedef	enum	{
 #define      M_PREPEND	0x0080
 #define      M_HELP	0x0100
 #define      M_WHATIS	0x0200
+#define      M_NONPERSIST	0x0400
 
 /**
  **  markers for switching 
@@ -363,27 +390,27 @@ typedef	enum	{
  **/
 
 #ifndef UNAME_SYSNAME
-#define UNAME_SYSNAME	"unknown"
+#  define UNAME_SYSNAME		"unknown"
 #endif
 
 #ifndef UNAME_NODENAME
-#define UNAME_NODENAME	"unknown"
+#  define UNAME_NODENAME	"unknown"
 #endif
 
 #ifndef UNAME_RELEASE
-#define UNAME_RELEASE	"unknown"
+#  define UNAME_RELEASE		"unknown"
 #endif
 
 #ifndef UNAME_VERSION
-#define UNAME_VERSION	"unknown"
+#  define UNAME_VERSION		"unknown"
 #endif
 
 #ifndef UNAME_MACHINE
-#define UNAME_MACHINE	"unknown"
+#  define UNAME_MACHINE		"unknown"
 #endif
 
 #ifndef UNAME_DOMAIN
-#define UNAME_DOMAIN	"unknown"
+#  define UNAME_DOMAIN		"unknown"
 #endif
 
 /**
@@ -391,19 +418,19 @@ typedef	enum	{
  **/
 
 #ifndef	RCFILE
-#define	RCFILE		"rc"
+#  define	RCFILE		"rc"
 #endif
 
 #ifndef	MODULERCFILE
-#define	MODULERCFILE	".modulerc"
+#  define	MODULERCFILE	".modulerc"
 #endif
 
 #ifndef	VERSIONFILE
-#define	VERSIONFILE	".version"
+#  define	VERSIONFILE	".version"
 #endif
 
 #ifndef	APR_CACHE
-#define	APR_CACHE	"apropos.cache"
+#  define	APR_CACHE	"apropos.cache"
 #endif
 
 /**
@@ -418,31 +445,33 @@ typedef	enum	{
 /** ************************************************************************ **/
 
 /**
- **  I'm going to remove all of the calls to free( ) since they arn't
- **    very necessary for Modules.  Since the modulecmd program is only run for
+ **  I'm going to remove all of the calls to free( ) since they are not
+ **    necessary for Modules.  Since the modulecmd program is only run for
  **    a very short time ( usually <1sec) it's faster to not clutter the heap
  **    by freeing up memory.
  **
  **  If you disagree with this decision, or have some problems with this
  **    behavior on your system, configure with --enable-free
+ **
+ **  Note that all memory deallocations should go through null_free()
  **/
 
 #ifndef USE_FREE
-#define  free( x)  
-#define  FreeList( x, y)  
+#  define  free( x)  
+#  define  FreeList( x, y)  
 #endif
 
 /** 
- **  Some systems don't defined S_ISDIR and S_ISREG
+ **  Some systems don't define S_ISDIR and S_ISREG
  **/
 
 #ifdef HAVE_SYS_STAT_H
-#ifndef S_ISDIR
-#define S_ISDIR( m) (((m) & S_IFMT) == S_IFDIR)
-#endif
-#ifndef S_ISREG
-#define S_ISREG( m) (((m) & S_IFMT) == S_IFREG)
-#endif
+#  ifndef S_ISDIR
+#    define S_ISDIR( m) (((m) & S_IFMT) == S_IFDIR)
+#  endif
+#  ifndef S_ISREG
+#    define S_ISREG( m) (((m) & S_IFMT) == S_IFREG)
+#  endif
 #endif
 
 /**
@@ -459,17 +488,22 @@ typedef	enum	{
 extern	char	**environ;
 
 extern	char	 *version_string;
+extern	char	 *date_string;
 extern	char	 *g_current_module;
-extern	char	 *specified_module;
-extern	char	**shell_startups;
-extern	char	  shell_name[];
-extern	char	  shell_derelict[];
+extern	char	 *g_specified_module;
+extern	char	 *shell_name;
+extern	char	 *shell_derelict;
+extern	char	 *shell_init;
+extern	char	 *shell_cmd_separator;
 extern	int	  g_flags;
+extern	int	  g_retval;
+extern	int	  g_output;
 extern	int	  append_flag;
 extern	char	 *line;
 extern	char	 *error_line;
 extern	char	  local_line[];
 extern	char	  _default[];
+extern	char	  _colon[];
 
 extern	int	  linenum;
 
@@ -489,6 +523,7 @@ extern	char	*purgeRE;
 extern	char	*clearRE;
 extern	char	*whatisRE;
 extern	char	*aproposRE;
+extern	char	*refreshRE;
 
 extern	Tcl_HashTable	*setenvHashTable;
 extern	Tcl_HashTable	*unsetenvHashTable;
@@ -530,13 +565,16 @@ extern	char	 long_header[];
 /**  locate_module.c  **/
 extern	int	  Locate_ModuleFile( Tcl_Interp*, char*, char*, char*);
 extern	char	**SortedDirList( Tcl_Interp*, char*, char*, int*);
-extern	char	**SplitIntoList( Tcl_Interp*, char*, int*);
+extern	char	**SplitIntoList( Tcl_Interp*, char*, int*, const char*);
 extern	int	  SourceVers( Tcl_Interp*, char*, char*);
-extern	int	  SourceRC( Tcl_Interp *interp, char *path, char *name);
+extern	int	  SourceRC( Tcl_Interp *interp, char *, char *);
 #ifdef USE_FREE
 extern	void	  FreeList( char**, int);
 #endif
 
+/**  main.c  **/
+extern  void	  module_usage(FILE *output);
+
 /**  ModuleCmd_Avail.c  **/
 extern	int	  ModuleCmd_Avail( Tcl_Interp*, int, char*[]);
 extern	void	  print_dirents( char*);
@@ -546,7 +584,7 @@ extern	int	  check_dir( char*);
 extern	fi_ent	 *get_dir( char*, char*, int*, int*);
 extern	void	  dirlst_to_list( char**, fi_ent*, int, int*, char*, char*);
 extern	void	  delete_dirlst( fi_ent*, int);
-extern	void	  delete_cache_list( char** list, int count);
+extern	void	  delete_cache_list( char**, int);
 
 /**  ModuleCmd_Clear.c  **/
 extern	int	  ModuleCmd_Clear( Tcl_Interp*, int, char*[]);
@@ -584,72 +622,72 @@ extern	int	  ModuleCmd_Use( Tcl_Interp*, int, char*[]);
 extern	int	  ModuleCmd_UnUse( Tcl_Interp*, int, char*[]);
 
 /**  cmdAlias.c  **/
-extern	int	  cmdSetAlias( ClientData, Tcl_Interp*, int, char*[]);
+extern	int	  cmdSetAlias( ClientData, Tcl_Interp*, int, CONST84 char*[]);
 
 /**  cmdConflict.c  **/
-extern	int	  cmdConflict( ClientData, Tcl_Interp*, int, char*[]);
-extern	int	  cmdPrereq( ClientData, Tcl_Interp*, int, char*[]);
+extern	int	  cmdConflict( ClientData, Tcl_Interp*, int, CONST84 char*[]);
+extern	int	  cmdPrereq( ClientData, Tcl_Interp*, int, CONST84 char*[]);
 
 /**  cmdIsLoaded.c  **/
-extern        int       cmdIsLoaded(ClientData, Tcl_Interp*, int, char*[]);
+extern	int	  cmdIsLoaded(ClientData, Tcl_Interp*, int, CONST84 char*[]);
 
 /**  cmdVerbose.c  **/
-extern	int	  cmdModuleVerbose( ClientData, Tcl_Interp*, int, char*[]);
+extern	int	  cmdModuleVerbose(ClientData,Tcl_Interp*,int,CONST84 char*[]);
 
 /**  cmdWhatis.c  **/
 extern	char	**whatis;
 extern	void	  cmdModuleWhatisInit(void);
 extern	void	  cmdModuleWhatisShut(void);
-extern	int	  cmdModuleWhatis( ClientData, Tcl_Interp*, int, char*[]);
+extern	int	  cmdModuleWhatis(ClientData,Tcl_Interp*,int,CONST84 char*[]);
 
 /**  cmdInfo.c  **/
-extern	int	  cmdModuleInfo( ClientData, Tcl_Interp*, int, char*[]);
+extern	int	  cmdModuleInfo(ClientData, Tcl_Interp*, int, CONST84 char*[]);
 extern	char	 *module_command;
 
 /**  cmdMisc.c  **/
-extern	int	  cmdSystem( ClientData, Tcl_Interp*, int, char*[]);
+extern	int	  cmdSystem( ClientData, Tcl_Interp*, int, CONST84 char*[]);
 
 /**  cmdModule.c  **/
-extern	int	  cmdModule( ClientData, Tcl_Interp*, int, char*[]);
+extern	int	  cmdModule( ClientData, Tcl_Interp*, int, CONST84 char*[]);
 extern	int	  Read_Modulefile( Tcl_Interp*, char*);
 extern	int	  Execute_TclFile( Tcl_Interp*, char*);
 extern	int	  CallModuleProcedure( Tcl_Interp*, Tcl_DString*, char*, char*,
 			int);
 
 /**  cmdPath.c  **/
-extern	int	  cmdSetPath( ClientData, Tcl_Interp*, int, char*[]);
-extern	int	  cmdRemovePath( ClientData, Tcl_Interp*, int, char*[]);
+extern	int	  cmdSetPath( ClientData, Tcl_Interp*, int, CONST84 char*[]);
+extern	int	  cmdRemovePath( ClientData, Tcl_Interp*, int, CONST84 char*[]);
 extern	char	 *chk_nullvars( char*);
 
 /**  cmdSetenv.c  **/
-extern	int	  cmdSetEnv( ClientData, Tcl_Interp*, int, char*[]);
+extern	int	  cmdSetEnv( ClientData, Tcl_Interp*, int, CONST84 char*[]);
 extern	int	  moduleSetenv( Tcl_Interp*, char	 *, char*, int);
-extern	int	  cmdUnsetEnv( ClientData, Tcl_Interp*, int, char*[]);
+extern	int	  cmdUnsetEnv( ClientData, Tcl_Interp*, int, CONST84 char*[]);
 extern	int	  moduleUnsetenv( Tcl_Interp*, char	 *);
 
 /**  cmdUname.c  **/
-extern	int	  cmdUname( ClientData, Tcl_Interp*, int, char*[]);
+extern	int	  cmdUname( ClientData, Tcl_Interp*, int, CONST84 char*[]);
 
 /**  cmdXResource.c  **/
-extern	int	  cmdXResource( ClientData, Tcl_Interp*, int, char*[]);
+extern	int	  cmdXResource( ClientData, Tcl_Interp*, int, CONST84 char*[]);
 extern	void	  xresourceFinish( int);
 
 /**  cmdUlvl.c  **/
-extern	int	  cmdModuleUser( ClientData, Tcl_Interp*, int, char*[]);
+extern	int	  cmdModuleUser(ClientData, Tcl_Interp*, int, CONST84 char*[]);
 extern	int	  cmdModuleUser_sub( char *user_level);
 
 /**  cmdLog.c  **/
-extern	int	  cmdModuleLog( ClientData, Tcl_Interp*, int, char*[]);
+extern	int	  cmdModuleLog( ClientData, Tcl_Interp*, int, CONST84 char*[]);
 
 /**  cmdTrace.c  **/
-extern	int	  cmdModuleTrace( ClientData, Tcl_Interp*, int, char*[]);
+extern	int	  cmdModuleTrace(ClientData,Tcl_Interp*, int, CONST84 char*[]);
 extern	char	 *GetTraceSel(Tcl_Interp*, char*);
 extern	int	  CheckTracing(Tcl_Interp*, char*, char*);
 extern	int	  CheckTracingList(Tcl_Interp*, char*, int, char**);
 
 /**  cmdVersion.c  **/
-extern	int	  cmdModuleVersion( ClientData, Tcl_Interp*, int, char*[]);
-extern	int	  cmdModuleAlias( ClientData, Tcl_Interp*, int, char*[]);
+extern	int	  cmdModuleVersion(ClientData,Tcl_Interp*,int,CONST84 char*[]);
+extern	int	  cmdModuleAlias(ClientData,Tcl_Interp*, int, CONST84 char*[]);
 extern	int	  AliasLookup( char*, char**, char**);
 extern	int	  VersionLookup( char*, char**, char**);
 extern	char	 *ExpandVersions( char*);
@@ -658,7 +696,7 @@ extern	char	 *ExpandVersions( char*);
 extern	int	  Initialize_Tcl( Tcl_Interp**, int, char*[], char*[]);
 extern	int	  InitializeModuleCommands( Tcl_Interp*);
 extern	int	  Setup_Environment( Tcl_Interp*);
-extern	int	  SetStartupFiles( void);
+extern	char	**SetStartupFiles( char *shell_name);
 extern	int	  TieStdout( void);
 extern	int	  UnTieStdout( int);
 
@@ -674,34 +712,40 @@ extern	void	  Delete_Hash_Tables( Tcl_HashTable**);
 extern	Tcl_HashTable** Copy_Hash_Tables( void);
 extern	int	  Unwind_Modulefile_Changes( Tcl_Interp*, Tcl_HashTable**);
 extern	int	  Output_Modulefile_Changes( Tcl_Interp*);
-extern	char	 *set_derelict( const char*);
 extern	int	  store_env( void);
 extern	int	  free_stored_env( void);
-extern	void	  set_marked_entry( Tcl_HashTable*, char*, int);
-extern	int	  chk_marked_entry( Tcl_HashTable*, char*);
+extern	void	  set_marked_entry( Tcl_HashTable*, char*, intptr_t);
+extern	intptr_t  chk_marked_entry( Tcl_HashTable*, char*);
 extern	Tcl_HashTable*  environ_changes;
 extern	Tcl_HashTable*  alias_changes;
 extern	int	  IsLoaded( Tcl_Interp*, char*, char**, char*);
 extern	int	  IsLoaded_ExactMatch( Tcl_Interp*, char*, char	**, char*);
 extern	int	  Update_LoadedList( Tcl_Interp*, char*, char*);
-extern	int	  ForceBasePath( Tcl_Interp*, char*, char*);
-extern	int	  ForceSacredPath( Tcl_Interp*, char*, char*);
 extern	int	  check_magic( char*, char*, int);
+extern	char	 *xstrtok_r(char *, const char *, char **);
+extern	char	 *xstrtok(char *, const char *);
 extern	void	  chk4spch( char*);
 extern	void	  cleanse_path( const char*, char*, int);
 extern	char	 *xdup(char const *);
 extern	char	 *xgetenv(char const *);
+extern  int       tmpfile_mod( char**, FILE**);
+extern	char	 *stringer(char *, int, ...);
+extern	void	  null_free(void **);
+extern	size_t	  countTclHash(Tcl_HashTable *);
+extern	EM_RetVal	ReturnValue( Tcl_Interp*, int);
+extern	void	  OutputExit();
 
 #ifndef HAVE_STRDUP
-extern	char	 * strdup( char*);
+#  undef strdup
+extern	char	 *strdup( char*);
 #endif
 
 #ifndef HAVE_STRTOK
-extern	char *strtok( char *, const char *);
+extern	char	 *strtok( char *, const char *);
 #endif
 
 /** error.c **/
-extern	char	**GetFacilityPtr( char *facility);
+extern	char	**GetFacilityPtr( char *);
 extern	int 	  Module_Error(	ErrType, char*, int, ...);
 extern	int	  CheckFacility( char*, int*, int*);
 extern	void	  Module_Tracing( int, int, char**);
